//
//  BugTableViewController.m
//  ScaryBugs
//
//  Created by Main Account on 2/12/14.
//  Copyright (c) 2014 Razeware LLC. All rights reserved.
//

#import "BugTableViewController.h"
#import "ScaryBug.h"
#import "BugSection.h"
#import "ScaryBugCell.h"
#import "EditViewController.h"

@interface BugTableViewController ()

@property (strong, nonatomic) NSMutableArray *bugSections;
@end

@implementation BugTableViewController

- (void)setupBugs {

  // Create sections
  self.bugSections = [NSMutableArray arrayWithCapacity:5];
  [self.bugSections addObject:[[BugSection alloc] initWithHowScary:ScaryFactorNotScary]];
  [self.bugSections addObject:[[BugSection alloc] initWithHowScary:ScaryFactorALittleScary]];
  [self.bugSections addObject:[[BugSection alloc] initWithHowScary:ScaryFactorAverageScary]];
  [self.bugSections addObject:[[BugSection alloc] initWithHowScary:ScaryFactorQuiteScary]];
  [self.bugSections addObject:[[BugSection alloc] initWithHowScary:ScaryFactorAiiiiieeeee]];
  
  // Sort bugs into sections
  NSMutableArray *bugs = [ScaryBug bugs];
  for (ScaryBug *bug in bugs) {
    BugSection *section = self.bugSections[(int) bug.howScary];
    [section.bugs addObject:bug];
  }

}

- (void)viewDidLoad
{
  [super viewDidLoad];
  [self setupBugs];
  self.navigationItem.rightBarButtonItem = self.editButtonItem;
  self.tableView.allowsSelectionDuringEditing = YES;
}

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView {
  return self.bugSections.count;
}

- (NSString *)tableView:(UITableView *)tableView titleForHeaderInSection:(NSInteger)section {
  BugSection *bugSection = self.bugSections[section];
  return [ScaryBug scaryFactorToString:bugSection.howScary];
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section
{
  int adjustment = [self isEditing] ? 1 : 0;
  
  BugSection *bugSection = self.bugSections[section];
  return bugSection.bugs.count + adjustment;
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath
{
  UITableViewCell *cell;
  
  BugSection *bugSection = self.bugSections[indexPath.section];
  if (indexPath.row >= bugSection.bugs.count && [self isEditing]) {
    cell = [tableView dequeueReusableCellWithIdentifier:@"NewRowCell" forIndexPath:indexPath];
    cell.textLabel.text = @"Add Bug";
    cell.detailTextLabel.text = nil;
  } else {
    cell = [tableView dequeueReusableCellWithIdentifier:@"BugCell" forIndexPath:indexPath];
    ScaryBugCell *bugCell = (ScaryBugCell *)cell;
    ScaryBug *bug = bugSection.bugs[indexPath.row];
    bugCell.bugImageView.image = bug.image;
    bugCell.bugNameLabel.text = bug.name;
    if (bug.howScary > ScaryFactorAverageScary) {
      bugCell.howScaryImageView.image = [UIImage imageNamed:@"shockedface2_full"];
    } else {
      bugCell.howScaryImageView.image = [UIImage imageNamed:@"shockedface2_empty"];
    }
  }
  
  return cell;
}

- (void)tableView:(UITableView *)tableView commitEditingStyle:(UITableViewCellEditingStyle)editingStyle forRowAtIndexPath:(NSIndexPath *)indexPath {
  // 1
  if (editingStyle == UITableViewCellEditingStyleDelete) {
    // 2
    BugSection *bugSection = self.bugSections[indexPath.section];
    // 3
    [bugSection.bugs removeObjectAtIndex:indexPath.row];
    // 4
    [tableView deleteRowsAtIndexPaths:@[indexPath] withRowAnimation:UITableViewRowAnimationAutomatic];
    // 5
  }
  else if (editingStyle == UITableViewCellEditingStyleInsert)
  {
    BugSection *bugSection = self.bugSections[indexPath.section];
    ScaryBug *newBug = [[ScaryBug alloc] initWithName:@"New Bug" image:nil howScary:bugSection.howScary];
    [bugSection.bugs addObject:newBug];
    [tableView insertRowsAtIndexPaths:@[indexPath] withRowAnimation:UITableViewRowAnimationAutomatic];
  }
}

- (void)setEditing:(BOOL)editing animated:(BOOL)animated {
  
  // 1
  [super setEditing:editing animated:animated];

  // 2
  if (editing) {
    // 3
    [self.tableView beginUpdates];
    // 4
    for (int i = 0; i < _bugSections.count; ++i) {
      BugSection *section = _bugSections[i];
      // 5
      [self.tableView insertRowsAtIndexPaths:@[[NSIndexPath indexPathForRow:section.bugs.count inSection:i]] withRowAnimation:UITableViewRowAnimationAutomatic];
    }
    // 6
    [self.tableView endUpdates];
  } else {
    // 7
    [self.tableView beginUpdates];
    for (int i = 0; i < _bugSections.count; ++i) {
      BugSection *section = _bugSections[i];
      [self.tableView deleteRowsAtIndexPaths:@[[NSIndexPath indexPathForRow:section.bugs.count inSection:i]] withRowAnimation:UITableViewRowAnimationAutomatic];
    }
    [self.tableView endUpdates];
  }
}

- (UITableViewCellEditingStyle)tableView:(UITableView *)tableView editingStyleForRowAtIndexPath:(NSIndexPath *)indexPath {
  
  BugSection *bugSection = self.bugSections[indexPath.section];
  
  if (indexPath.row >= bugSection.bugs.count) {
    return UITableViewCellEditingStyleInsert;
  } else {
    return UITableViewCellEditingStyleDelete;
  }
  
}

- (NSIndexPath *)tableView:(UITableView *)tableView willSelectRowAtIndexPath:(NSIndexPath *)indexPath {
  
  BugSection *bugSection = self.bugSections[indexPath.section];
  if ([self isEditing] && indexPath.row < bugSection.bugs.count) {
    return nil;
  }
  return indexPath;
}

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath {

  [self.tableView deselectRowAtIndexPath:indexPath animated:YES];
  
  BugSection *bugSection = self.bugSections[indexPath.section];
  if (indexPath.row >= bugSection.bugs.count && [self isEditing]) {
    [self tableView:tableView commitEditingStyle:UITableViewCellEditingStyleInsert forRowAtIndexPath:indexPath];
  }
}

- (void)tableView:(UITableView *)tableView moveRowAtIndexPath:(NSIndexPath *)sourceIndexPath toIndexPath:(NSIndexPath *)destinationIndexPath {

  // 1
  BugSection *sourceSection = [self.bugSections objectAtIndex:sourceIndexPath.section];
  ScaryBug *bugToMove = [sourceSection.bugs objectAtIndex:sourceIndexPath.row];
  BugSection *destSection = [self.bugSections objectAtIndex:destinationIndexPath.section];
  
  // 2
  if (sourceSection == destSection) {
    [destSection.bugs exchangeObjectAtIndex:destinationIndexPath.row withObjectAtIndex:sourceIndexPath.row];
  }
  // 3
  else {
    // 4
    bugToMove.howScary = destSection.howScary;
    [destSection.bugs insertObject:bugToMove atIndex:destinationIndexPath.row];
    [sourceSection.bugs removeObjectAtIndex:sourceIndexPath.row];
    
    double delayInSeconds = 0.2;
    dispatch_time_t popTime = dispatch_time(DISPATCH_TIME_NOW, (int64_t)(delayInSeconds * NSEC_PER_SEC));
    dispatch_after(popTime, dispatch_get_main_queue(), ^(void){
      [self.tableView reloadRowsAtIndexPaths:@[destinationIndexPath] withRowAnimation:UITableViewRowAnimationNone];
    });
  }
}

- (NSIndexPath *)tableView:(UITableView *)tableView targetIndexPathForMoveFromRowAtIndexPath:(NSIndexPath *)sourceIndexPath toProposedIndexPath:(NSIndexPath *)proposedDestinationIndexPath
{
  BugSection *bugSection = self.bugSections[proposedDestinationIndexPath.section];
  if (proposedDestinationIndexPath.row >= bugSection.bugs.count) {
    return [NSIndexPath indexPathForRow:bugSection.bugs.count-1 inSection:proposedDestinationIndexPath.section];
  } else {
    return proposedDestinationIndexPath;
  }
}

- (BOOL)tableView:(UITableView *)tableView canMoveRowAtIndexPath:(NSIndexPath *)indexPath {
  BugSection *bugSection = self.bugSections[indexPath.section];
  if (indexPath.row >= bugSection.bugs.count && [self isEditing]) {
    return NO;
  } else {
    return YES;
  }
}

- (CGFloat)tableView:(UITableView *)tableView heightForRowAtIndexPath:(NSIndexPath *)indexPath {
  BugSection *bugSection = self.bugSections[indexPath.section];
  if (indexPath.row >= bugSection.bugs.count && [self isEditing]) {
    return 44;
  } else {
    return 60;
  }
}

- (void)prepareForSegue:(UIStoryboardSegue *)segue sender:(id)sender {
  if ([segue.identifier isEqualToString:@"GoToEdit"]) {
    EditViewController *edit = (EditViewController *)segue.destinationViewController;
    NSIndexPath *indexPath = [self.tableView indexPathForCell:sender];
    BugSection *section = self.bugSections[indexPath.section];
    ScaryBug *bug = section.bugs[indexPath.row];
    edit.bug = bug;
  }
}

- (void)viewWillAppear:(BOOL)animated {
  [self.tableView reloadData];
  
//  int x = 0;
//  int y = 5 / x;
//  NSLog(@"%d", y);
  
}

@end
